@@ -130,7 +130,7 @@ class AzureRMMariaDbFirewallRuleInfo(AzureRMModuleBase):
         self.resource_group = None
         self.server_name = None
         self.name = None
-        super(AzureRMMariaDbFirewallRuleInfo, self).__init__(self.module_arg_spec, supports_tags=False)
+        super(AzureRMMariaDbFirewallRuleInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)
 
     def exec_module(self, **kwargs):
         is_old_facts = self.module._name == 'azure_rm_mariadbfirewallrule_facts'
