@@ -11,12 +11,12 @@ DOCUMENTATION = '''
 ---
 module: docker_image
 
-short_description: Manage docker images.
+short_description: Manage docker images
 
 
 description:
-  - Build, load or pull an image, making the image available for creating containers. Also supports tagging an
-    image into a repository and archiving an image to a C(.tar) file.
+  - Build, load or pull an image, making the image available for creating containers. Also supports tagging
+    an image, pushing an image, and archiving an image to a C(.tar) file.
 
 options:
   source:
@@ -157,7 +157,8 @@ options:
     description:
       - "Image name. Name format will be one of: C(name), C(repository/name), C(registry_server:port/name).
         When pushing or pulling an image the name can optionally include the tag by appending C(:tag_name)."
-      - Note that image IDs (hashes) are only supported for I(state=absent), and for I(state=present) with I(source=load).
+      - Note that image IDs (hashes) are only supported for I(state=absent), for I(state=present) with I(source=load),
+        and for I(state=present) with I(source=local).
     type: str
     required: yes
   pull:
@@ -179,8 +180,10 @@ options:
     default: no
   repository:
     description:
-      - Full path to a repository. Use with state C(present) to tag the image into the repository. Expects
-        format I(repository:tag). If no tag is provided, will use the value of the C(tag) parameter or I(latest).
+      - Use with state C(present) to tag the image.
+      - Expects format C(repository:tag). If no tag is provided, will use the value of the I(tag) parameter or C(latest).
+      - If I(push=true), I(repository) must either include a registry, or will be assumed to belong to the default
+        registry (Docker Hub).
     type: str
   state:
     description:
@@ -321,11 +324,8 @@ stdout:
 
 import errno
 import os
-import re
 import traceback
 
-from distutils.version import LooseVersion
-
 from ansible_collections.community.docker.plugins.module_utils.common import (
     clean_dict_booleans_for_docker_api,
     docker_version,
@@ -335,7 +335,9 @@ from ansible_collections.community.docker.plugins.module_utils.common import (
     is_valid_tag,
     RequestException,
 )
-from ansible.module_utils._text import to_native
+from ansible.module_utils.common.text.converters import to_native
+
+from ansible_collections.community.docker.plugins.module_utils.version import LooseVersion
 
 if docker_version is not None:
     try:
@@ -501,7 +503,7 @@ class ImageManager(DockerBaseClass):
                     # If the image vanished while we were trying to remove it, don't fail
                     pass
                 except Exception as exc:
-                    self.fail("Error removing image %s - %s" % (name, str(exc)))
+                    self.fail("Error removing image %s - %s" % (name, to_native(exc)))
 
             self.results['changed'] = True
             self.results['actions'].append("Removed image %s" % (name))
@@ -536,7 +538,7 @@ class ImageManager(DockerBaseClass):
             try:
                 saved_image = self.client.get_image(image_name)
             except Exception as exc:
-                self.fail("Error getting image %s - %s" % (image_name, str(exc)))
+                self.fail("Error getting image %s - %s" % (image_name, to_native(exc)))
 
             try:
                 with open(self.archive_path, 'wb') as fd:
@@ -547,7 +549,7 @@ class ImageManager(DockerBaseClass):
                         for chunk in saved_image.stream(2048, decode_content=False):
                             fd.write(chunk)
             except Exception as exc:
-                self.fail("Error writing image archive %s - %s" % (self.archive_path, str(exc)))
+                self.fail("Error writing image archive %s - %s" % (self.archive_path, to_native(exc)))
 
         if image:
             self.results['image'] = image
@@ -587,14 +589,14 @@ class ImageManager(DockerBaseClass):
                             changed = True
                     self.results['changed'] = changed
                 except Exception as exc:
-                    if re.search('unauthorized', str(exc)):
-                        if re.search('authentication required', str(exc)):
+                    if 'unauthorized' in str(exc):
+                        if 'authentication required' in str(exc):
                             self.fail("Error pushing image %s/%s:%s - %s. Try logging into %s first." %
-                                      (registry, repo_name, tag, str(exc), registry))
+                                      (registry, repo_name, tag, to_native(exc), registry))
                         else:
                             self.fail("Error pushing image %s/%s:%s - %s. Does the repository exist?" %
                                       (registry, repo_name, tag, str(exc)))
-                    self.fail("Error pushing image %s: %s" % (repository, str(exc)))
+                    self.fail("Error pushing image %s: %s" % (repository, to_native(exc)))
                 self.results['image'] = self.client.find_image(name=repository, tag=tag)
                 if not self.results['image']:
                     self.results['image'] = dict()
@@ -620,21 +622,21 @@ class ImageManager(DockerBaseClass):
         self.log("image %s was %s" % (repo, found))
 
         if not image or self.force_tag:
-            self.log("tagging %s:%s to %s:%s" % (name, tag, repo, repo_tag))
+            image_name = name
+            if not is_image_name_id(name) and tag and not name.endswith(':' + tag):
+                image_name = "%s:%s" % (name, tag)
+            self.log("tagging %s to %s:%s" % (image_name, repo, repo_tag))
             self.results['changed'] = True
-            self.results['actions'].append("Tagged image %s:%s to %s:%s" % (name, tag, repo, repo_tag))
+            self.results['actions'].append("Tagged image %s to %s:%s" % (image_name, repo, repo_tag))
             if not self.check_mode:
                 try:
                     # Finding the image does not always work, especially running a localhost registry. In those
                     # cases, if we don't set force=True, it errors.
-                    image_name = name
-                    if tag and not re.search(tag, name):
-                        image_name = "%s:%s" % (name, tag)
                     tag_status = self.client.tag(image_name, repo, tag=repo_tag, force=True)
                     if not tag_status:
                         raise Exception("Tag operation failed.")
                 except Exception as exc:
-                    self.fail("Error: failed to tag image - %s" % str(exc))
+                    self.fail("Error: failed to tag image - %s" % to_native(exc))
                 self.results['image'] = self.client.find_image(name=repo, tag=repo_tag)
                 if image and image['Id'] == self.results['image']['Id']:
                     self.results['changed'] = False
@@ -763,10 +765,10 @@ class ImageManager(DockerBaseClass):
                         )
         except EnvironmentError as exc:
             if exc.errno == errno.ENOENT:
-                self.client.fail("Error opening image %s - %s" % (self.load_path, str(exc)))
-            self.client.fail("Error loading image %s - %s" % (self.name, str(exc)), stdout='\n'.join(load_output))
+                self.client.fail("Error opening image %s - %s" % (self.load_path, to_native(exc)))
+            self.client.fail("Error loading image %s - %s" % (self.name, to_native(exc)), stdout='\n'.join(load_output))
         except Exception as exc:
-            self.client.fail("Error loading image %s - %s" % (self.name, str(exc)), stdout='\n'.join(load_output))
+            self.client.fail("Error loading image %s - %s" % (self.name, to_native(exc)), stdout='\n'.join(load_output))
 
         # Collect loaded images
         if has_output:
@@ -908,9 +910,11 @@ def main():
         ImageManager(client, results)
         client.module.exit_json(**results)
     except DockerException as e:
-        client.fail('An unexpected docker error occurred: {0}'.format(e), exception=traceback.format_exc())
+        client.fail('An unexpected docker error occurred: {0}'.format(to_native(e)), exception=traceback.format_exc())
     except RequestException as e:
-        client.fail('An unexpected requests error occurred when docker-py tried to talk to the docker daemon: {0}'.format(e), exception=traceback.format_exc())
+        client.fail(
+            'An unexpected requests error occurred when docker-py tried to talk to the docker daemon: {0}'.format(to_native(e)),
+            exception=traceback.format_exc())
 
 
 if __name__ == '__main__':
