@@ -693,7 +693,7 @@ class ApiParameters(Parameters):
     @property
     def state(self):
         if (self._values['state'] in ['user-up', 'unchecked', 'fqdn-up-no-addr', 'fqdn-up', 'fqdn-down']
-           and self._values['session'] in ['user-enabled']):
+           and self._values['session'] in ['user-enabled', 'monitor-enabled']):
             return 'present'
         elif self._values['state'] in ['down', 'up', 'checking'] and self._values['session'] == 'monitor-enabled':
             # monitor-enabled + checking:
@@ -803,7 +803,7 @@ class UsableChanges(Changes):
         monitor_string = self._values['monitors']
         if monitor_string is None:
             return None
-        if '{' in monitor_string and '}':
+        if '{' in monitor_string and '}' in monitor_string:
             tmp = monitor_string.strip('}').split('{')
             monitor = ''.join(tmp).rstrip()
             return monitor
@@ -920,7 +920,6 @@ class Difference(object):
     def state(self):
         if self.want.state == self.have.state:
             return None
-        self.want.f
         if self.want.state == 'forced_offline':
             return {
                 'state': 'user-down',
@@ -1113,6 +1112,10 @@ class ModuleManager(object):
                 "Aggregates must be provided with both address and port."
             )
         delimiter = ':'
+        # If user provides us a name for the aggregate element, we use its name with port, this is
+        # how F5 BIG-IP behaves as well.
+        if 'name' in item and item['name']:
+            return '{0}{1}{2}'.format(item['name'], delimiter, item['port'])
         try:
             if validate_ip_v6_address(item['address']):
                 delimiter = '.'
@@ -1123,10 +1126,10 @@ class ModuleManager(object):
     def compare_addresses(self, items):
         if any('address' in item for item in items):
             aggregates = [self._join_address_port(item) for item in items if 'address' in item and item['address']]
-            collection = [member['address'] for member in self.on_device]
+            collection = [member['name'] for member in self.on_device]
             diff = set(collection) - set(aggregates)
             if diff:
-                addresses = [item['selfLink'] for item in self.on_device if item['address'] in diff]
+                addresses = [item['selfLink'] for item in self.on_device if item['name'] in diff]
                 self.purge_links.extend(addresses)
                 return True
             return False
