@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# (c) 2021, NetApp, Inc
+# (c) 2022, NetApp, Inc
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
 '''
@@ -20,7 +20,7 @@ version_added: '21.4.0'
 author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
 
 description:
-- Create or delete Cloud Manager CVO for GCP.
+- Create, delete, or manage Cloud Manager CVO for GCP.
 
 options:
 
@@ -49,6 +49,12 @@ options:
     choices: ['GCP']
     type: str
 
+  gcp_encryption_parameters:
+    description:
+    - The GCP encryption parameters.
+    type: str
+    version_added: 21.10.0
+
   enable_compliance:
     description:
     - Enable the Cloud Compliance service on the working environment.
@@ -116,13 +122,32 @@ options:
   license_type:
     description:
     - The type of license to use.
-    - Single node ['gcp-cot-explore-paygo', 'gcp-cot-standard-paygo', 'gcp-cot-premium-paygo', 'gcp-cot-premium-byol'].
-    - HA ['gcp-ha-cot-explore-paygo', 'gcp-ha-cot-standard-paygo', 'gcp-ha-cot-premium-paygo', 'gcp-cot-premium-byol'].
-    choices: ['gcp-cot-standard-paygo', 'gcp-cot-explore-paygo', 'gcp-cot-premium-paygo', 'gcp-cot-premium-byol',
-    'gcp-ha-cot-standard-paygo', 'gcp-ha-cot-premium-paygo', 'gcp-ha-cot-explore-paygo', 'gcp-ha-cot-premium-byol']
+    - Single node ['gcp-cot-explore-paygo', 'gcp-cot-standard-paygo', 'gcp-cot-premium-paygo', 'gcp-cot-premium-byol', \
+     'capacity-paygo'].
+    - HA ['gcp-ha-cot-explore-paygo', 'gcp-ha-cot-standard-paygo', 'gcp-ha-cot-premium-paygo', 'gcp-cot-premium-byol', \
+     'ha-capacity-paygo'].
+    - Use capacity-paygo or ha-capacity-paygo for HA on selecting Bring Your Own License type Capacity-Based or Freemium.
+    - Use gcp-cot-premium-boyl or gcp-ha-cot-permium-boyl for HA on selecting Bring Your Own License type Node-Based.
+    choices: ['gcp-cot-standard-paygo', 'gcp-cot-explore-paygo', 'gcp-cot-premium-paygo', 'gcp-cot-premium-byol', \
+     'gcp-ha-cot-standard-paygo', 'gcp-ha-cot-premium-paygo', 'gcp-ha-cot-explore-paygo', 'gcp-ha-cot-premium-byol', \
+     'capacity-paygo', 'ha-capacity-paygo']
     type: str
     default: 'gcp-cot-standard-paygo'
 
+  provided_license:
+    description:
+    - Using a NLF license file for BYOL deployment
+    type: str
+
+  capacity_package_name:
+    description:
+    - Capacity package name is required when selecting a capacity based license.
+    - Essential only available with Bring Your Own License Capacity-Based.
+    - Professional available as an annual contract from a cloud provider or Bring Your Own License Capacity-Based.
+    choices: ['Professional', 'Essential', 'Freemium']
+    type: str
+    version_added: 21.12.0
+
   mediator_zone:
     description:
     - The zone for mediator.
@@ -228,6 +253,7 @@ options:
   svm_password:
     description:
     - The admin password for Cloud Volumes ONTAP.
+    - It will be updated on each run.
     type: str
 
   tier_level:
@@ -308,9 +334,9 @@ options:
   writing_speed_state:
     description:
     - The write speed setting for Cloud Volumes ONTAP ['NORMAL','HIGH'].
+    - Default value is 'NORMAL' for non-HA GCP CVO
     - This argument is not relevant for HA pairs.
     type: str
-    default: 'NORMAL'
 
   zone:
     description:
@@ -318,6 +344,22 @@ options:
     required: true
     type: str
 
+  upgrade_ontap_version:
+    description:
+    - Indicates whether to upgrade ONTAP image on the CVO.
+    - If the current version already matches the desired version, no action is taken.
+    type: bool
+    default: false
+    version_added: 21.13.0
+
+  update_svm_password:
+    description:
+    - Indicates whether to update svm_password on the CVO.
+    - When set to true, the module is not idempotent, as we cannot read the current password.
+    type: bool
+    default: false
+    version_added: 21.13.0
+
 notes:
 - Support check_mode.
 '''
@@ -405,7 +447,7 @@ from ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp import
 
 GCP_LICENSE_TYPES = ["gcp-cot-standard-paygo", "gcp-cot-explore-paygo", "gcp-cot-premium-paygo", "gcp-cot-premium-byol",
                      "gcp-ha-cot-standard-paygo", "gcp-ha-cot-premium-paygo", "gcp-ha-cot-explore-paygo",
-                     "gcp-ha-cot-premium-byol"]
+                     "gcp-ha-cot-premium-byol", "capacity-paygo", "ha-capacity-paygo"]
 GOOGLE_API_URL = "https://www.googleapis.com/compute/v1/projects"
 
 
@@ -420,6 +462,7 @@ class NetAppCloudManagerCVOGCP:
             capacity_tier=dict(required=False, type='str', choices=['cloudStorage']),
             client_id=dict(required=True, type='str'),
             data_encryption_type=dict(required=False, choices=['GCP'], type='str'),
+            gcp_encryption_parameters=dict(required=False, type='str', no_log=True),
             enable_compliance=dict(required=False, type='bool', default=False),
             firewall_rule=dict(required=False, type='str'),
             gcp_labels=dict(required=False, type='list', elements='dict', options=dict(
@@ -454,6 +497,8 @@ class NetAppCloudManagerCVOGCP:
             tier_level=dict(required=False, type='str', choices=['standard', 'nearline', 'coldline'],
                             default='standard'),
             use_latest_version=dict(required=False, type='bool', default=True),
+            capacity_package_name=dict(required=False, type='str', choices=['Professional', 'Essential', 'Freemium']),
+            provided_license=dict(required=False, type='str'),
             vpc_id=dict(required=True, type='str'),
             vpc0_firewall_rule_name=dict(required=False, type='str'),
             vpc0_node_and_data_connectivity=dict(required=False, type='str'),
@@ -464,53 +509,32 @@ class NetAppCloudManagerCVOGCP:
             vpc3_data_replication=dict(required=False, type='str'),
             vpc3_firewall_rule_name=dict(required=False, type='str'),
             workspace_id=dict(required=False, type='str'),
-            writing_speed_state=dict(required=False, type='str', default='NORMAL'),
+            writing_speed_state=dict(required=False, type='str'),
             zone=dict(required=True, type='str'),
+            upgrade_ontap_version=dict(required=False, type='bool', default=False),
+            update_svm_password=dict(required=False, type='bool', default=False),
         ))
 
         self.module = AnsibleModule(
             argument_spec=self.argument_spec,
             required_one_of=[['refresh_token', 'sa_client_id']],
             required_together=[['sa_client_id', 'sa_secret_key']],
+            required_if=[
+                ['license_type', 'capacity-paygo', ['capacity_package_name']],
+                ['license_type', 'ha-capacity-paygo', ['capacity_package_name']],
+            ],
             supports_check_mode=True
         )
         self.na_helper = NetAppModule()
         self.parameters = self.na_helper.set_parameters(self.module.params)
-
+        self.changeable_params = ['svm_password', 'tier_level', 'gcp_labels', 'ontap_version', 'instance_type']
         self.rest_api = CloudManagerRestAPI(self.module)
+        self.rest_api.url += self.rest_api.environment_data['CLOUD_MANAGER_HOST']
+        self.rest_api.api_root_path = '/occm/api/gcp/%s' % ('ha' if self.parameters['is_ha'] else 'vsa')
         self.headers = {
             'X-Agent-Id': self.rest_api.format_cliend_id(self.parameters['client_id'])
         }
 
-    def get_tenant(self):
-        """
-        Get workspace ID (tenant)
-        """
-
-        api_url = '%s/occm/api/tenants' % self.rest_api.environment_data['CLOUD_MANAGER_HOST']
-        response, error, dummy = self.rest_api.get(api_url, header=self.headers)
-        if error is not None:
-            self.module.fail_json(
-                msg="Error: unexpected response on getting tenant for gcp cvo: %s, %s" % (str(error), str(response)))
-
-        return response[0]['publicId']
-
-    def get_nss(self):
-        """
-        Get nss account
-        """
-
-        api_url = '%s/occm/api/accounts' % self.rest_api.environment_data['CLOUD_MANAGER_HOST']
-        response, error, dummy = self.rest_api.get(api_url, header=self.headers)
-        if error is not None:
-            self.module.fail_json(
-                msg="Error: unexpected response on getting nss for gcp cvo: %s, %s" % (str(error), str(response)))
-
-        if len(response['nssAccounts']) == 0:
-            self.module.fail_json(msg="could not find any NSS account")
-
-        return response['nssAccounts'][0]['publicId']
-
     @staticmethod
     def has_self_link(param):
         return param.startswith(("https://www.googleapis.com/compute/", "projects/"))
@@ -518,13 +542,25 @@ class NetAppCloudManagerCVOGCP:
     def create_cvo_gcp(self):
 
         if self.parameters.get('workspace_id') is None:
-            self.parameters['workspace_id'] = self.get_tenant()
+            response, msg = self.na_helper.get_tenant(self.rest_api, self.headers)
+            if response is None:
+                self.module.fail_json(msg)
+            self.parameters['workspace_id'] = response
 
         if self.parameters.get('nss_account') is None:
             if self.parameters.get('platform_serial_number') is not None:
                 if not self.parameters['platform_serial_number'].startswith('Eval-'):
                     if self.parameters['license_type'] == 'gcp-cot-premium-byol' or self.parameters['license_type'] == 'gcp-ha-cot-premium-byol':
-                        self.parameters['nss_account'] = self.get_nss()
+                        response, msg = self.na_helper.get_nss(self.rest_api, self.headers)
+                        if response is None:
+                            self.module.fail_json(msg)
+                        self.parameters['nss_account'] = response
+
+        if self.parameters.get('capacity_package_name') is not None:
+            if self.parameters['is_ha'] is True and self.parameters['license_type'] != 'ha-capacity-paygo':
+                self.module.fail_json(msg="license_type value must be ha-capacity-paygo")
+            if self.parameters['is_ha'] is False and self.parameters['license_type'] != 'capacity-paygo':
+                self.module.fail_json(msg="license_type value must be capacity-paygo")
 
         json = {"name": self.parameters['name'],
                 "region": self.parameters['zone'],
@@ -538,18 +574,28 @@ class NetAppCloudManagerCVOGCP:
                 "svmPassword": self.parameters['svm_password'],
                 "backupVolumesToCbs": self.parameters['backup_volumes_to_cbs'],
                 "enableCompliance": self.parameters['enable_compliance'],
-                "writingSpeedState": self.parameters['writing_speed_state'],
                 "vsaMetadata": {
                     "ontapVersion": self.parameters['ontap_version'],
                     "licenseType": self.parameters['license_type'],
+                    "useLatestVersion": self.parameters['use_latest_version'],
                     "instanceType": self.parameters['instance_type']}
                 }
 
-        if self.parameters.get('data_encryption_type'):
-            json.update({'dataEncryptionType': self.parameters['data_encryption_type']})
+        if self.parameters['is_ha'] is False:
+            if self.parameters.get('writing_speed_state') is None:
+                self.parameters['writing_speed_state'] = 'NORMAL'
+            json.update({'writingSpeedState': self.parameters['writing_speed_state']})
 
-        if self.parameters.get('use_latest_version') is not None:
-            json['vsaMetadata'].update({'useLatestVersion': self.parameters['use_latest_version']})
+        if self.parameters.get('data_encryption_type') is not None and self.parameters['data_encryption_type'] == "GCP":
+            json.update({'dataEncryptionType': self.parameters['data_encryption_type']})
+            if self.parameters.get('gcp_encryption_parameters') is not None:
+                json.update({"gcpEncryptionParameters": {"key": self.parameters['gcp_encryption_parameters']}})
+
+        if self.parameters.get('provided_license') is not None:
+            json['vsaMetadata'].update({"providedLicense": self.parameters['provided_license']})
+
+        if self.parameters.get('capacity_package_name') is not None:
+            json['vsaMetadata'].update({"capacityPackageName": self.parameters['capacity_package_name']})
 
         if self.parameters.get('project_id'):
             json.update({'project': self.parameters['project_id']})
@@ -682,69 +728,86 @@ class NetAppCloudManagerCVOGCP:
 
             json["haParams"] = ha_params
 
-        base_url = '/occm/api/gcp/vsa/working-environments'
-        if self.parameters.get('is_ha'):
-            base_url = '/occm/api/gcp/ha/working-environments'
-
-        api_url = '%s%s' % (self.rest_api.environment_data['CLOUD_MANAGER_HOST'], base_url)
+        api_url = '%s/working-environments' % self.rest_api.api_root_path
         response, error, on_cloud_request_id = self.rest_api.post(api_url, json, header=self.headers)
         if error is not None:
             self.module.fail_json(
                 msg="Error: unexpected response on creating cvo gcp: %s, %s" % (str(error), str(response)))
         working_environment_id = response['publicId']
-        wait_on_completion_api_url = '%s/occm/api/audit/activeTask/%s' % (self.rest_api.environment_data['CLOUD_MANAGER_HOST'], str(on_cloud_request_id))
+        wait_on_completion_api_url = '/occm/api/audit/activeTask/%s' % str(on_cloud_request_id)
         err = self.rest_api.wait_on_completion(wait_on_completion_api_url, "CVO", "create", 60, 60)
 
         if err is not None:
             self.module.fail_json(msg="Error: unexpected response wait_on_completion for creating CVO GCP: %s" % str(err))
         return working_environment_id
 
-    def get_working_environment(self):
-        """
-        Get working environment details including:
-        name: working environment name
-        """
-        api_url = '%s/occm/api/working-environments' % self.rest_api.environment_data['CLOUD_MANAGER_HOST']
-        response, error, dummy = self.rest_api.get(api_url, header=self.headers)
-        if error is not None:
-            self.module.fail_json(
-                msg="Error: unexpected response on getting gcp cvo: %s, %s" % (str(error), str(response)))
-
-        for each in response['gcpVsaWorkingEnvironments']:
-            if each['name'] == self.parameters['name']:
-                return each
-
-        return None
+    def update_cvo_gcp(self, working_environment_id, modify):
+        base_url = '%s/working-environments/%s/' % (self.rest_api.api_root_path, working_environment_id)
+        for item in modify:
+            if item == 'svm_password':
+                response, error = self.na_helper.update_svm_password(base_url, self.rest_api, self.headers, self.parameters['svm_password'])
+                if error is not None:
+                    self.module.fail_json(changed=False, msg=error)
+            if item == 'gcp_labels':
+                tag_list = None
+                if 'gcp_labels' in self.parameters:
+                    tag_list = self.parameters['gcp_labels']
+                response, error = self.na_helper.update_cvo_tags(base_url, self.rest_api, self.headers, 'gcp_labels', tag_list)
+                if error is not None:
+                    self.module.fail_json(changed=False, msg=error)
+            if item == 'tier_level':
+                response, error = self.na_helper.update_tier_level(base_url, self.rest_api, self.headers, self.parameters['tier_level'])
+                if error is not None:
+                    self.module.fail_json(changed=False, msg=error)
+            if item == 'ontap_version':
+                response, error = self.na_helper.upgrade_ontap_image(self.rest_api, self.headers, self.parameters['ontap_version'])
+                if error is not None:
+                    self.module.fail_json(changed=False, msg=error)
+            if item == 'instance_type' or item == 'license_type':
+                response, error = self.na_helper.update_instance_license_type(base_url, self.rest_api, self.headers,
+                                                                              self.parameters['instance_type'],
+                                                                              self.parameters['license_type'])
+                if error is not None:
+                    self.module.fail_json(changed=False, msg=error)
 
     def delete_cvo_gcp(self, we_id):
         """
         Delete GCP CVO
         """
-        self.na_helper.set_api_root_path(self.get_working_environment(), self.rest_api)
-        api_url = '%s%s/%s' % (self.rest_api.environment_data['CLOUD_MANAGER_HOST'], self.rest_api.api_root_path + '/working-environments', we_id)
+        api_url = '%s/working-environments/%s' % (self.rest_api.api_root_path, we_id)
         response, error, on_cloud_request_id = self.rest_api.delete(api_url, None, header=self.headers)
         if error is not None:
             self.module.fail_json(msg="Error: unexpected response on deleting cvo gcp: %s, %s" % (str(error), str(response)))
 
-        wait_on_completion_api_url = '%s/occm/api/audit/activeTask/%s' % (self.rest_api.environment_data['CLOUD_MANAGER_HOST'], str(on_cloud_request_id))
+        wait_on_completion_api_url = '/occm/api/audit/activeTask/%s' % str(on_cloud_request_id)
         err = self.rest_api.wait_on_completion(wait_on_completion_api_url, "CVO", "delete", 40, 60)
         if err is not None:
             self.module.fail_json(msg="Error: unexpected response wait_on_completion for deleting cvo gcp: %s" % str(err))
 
     def apply(self):
         working_environment_id = None
-        current = self.get_working_environment()
+        modify = None
+
+        current, dummy = self.na_helper.get_working_environment_details_by_name(self.rest_api, self.headers,
+                                                                                self.parameters['name'], "gcp")
+        if current:
+            self.parameters['working_environment_id'] = current['publicId']
         # check the action
         cd_action = self.na_helper.get_cd_action(current, self.parameters)
 
+        if current and self.parameters['state'] != 'absent':
+            working_environment_id = current['publicId']
+            modify, error = self.na_helper.is_cvo_update_needed(self.rest_api, self.headers, self.parameters, self.changeable_params, 'gcp')
+            if error is not None:
+                self.module.fail_json(changed=False, msg=error)
+
         if self.na_helper.changed and not self.module.check_mode:
             if cd_action == "create":
                 working_environment_id = self.create_cvo_gcp()
             elif cd_action == "delete":
                 self.delete_cvo_gcp(current['publicId'])
-
-        if self.parameters['state'] == "present" and cd_action is None:
-            self.module.warn(warning=self.parameters['name'] + ' already exists')
+            else:
+                self.update_cvo_gcp(current['publicId'], modify)
 
         self.module.exit_json(changed=self.na_helper.changed, working_environment_id=working_environment_id)
 
