@@ -16,9 +16,9 @@ ANSIBLE_METADATA = {
 DOCUMENTATION = r"""
 ---
 module: netbox_vrf
-short_description: Create, update or delete vrfs within Netbox
+short_description: Create, update or delete vrfs within NetBox
 description:
-  - Creates, updates or removes vrfs from Netbox
+  - Creates, updates or removes vrfs from NetBox
 notes:
   - Tags should be defined as a YAML list
   - This should be ran with connection C(local) and hosts C(localhost)
@@ -27,22 +27,9 @@ author:
 requirements:
   - pynetbox
 version_added: '0.1.0'
+extends_documentation_fragment:
+  - netbox.netbox.common
 options:
-  netbox_url:
-    description:
-      - URL of the Netbox instance resolvable by Ansible control host
-    required: true
-    type: str
-  netbox_token:
-    description:
-      - The token created within Netbox to authorize API access
-    required: true
-    type: str
-  cert:
-    description:
-      - Certificate path
-    required: false
-    type: raw
   data:
     type: dict
     description:
@@ -95,39 +82,20 @@ options:
         elements: raw
       custom_fields:
         description:
-          - must exist in Netbox
+          - must exist in NetBox
         required: false
         type: dict
     required: true
-  state:
-    description:
-      - Use C(present) or C(absent) for adding or removing.
-    choices: [ absent, present ]
-    default: present
-    type: str
-  query_params:
-    description:
-      - This can be used to override the specified values in ALLOWED_QUERY_PARAMS that is defined
-      - in plugins/module_utils/netbox_utils.py and provides control to users on what may make
-      - an object unique in their environment.
-    required: false
-    type: list
-    elements: str
-  validate_certs:
-    description:
-      - If C(no), SSL certificates will not be validated. This should only be used on personally controlled sites using self-signed certificates.
-    default: true
-    type: raw
 """
 
 EXAMPLES = r"""
-- name: "Test Netbox modules"
+- name: "Test NetBox modules"
   connection: local
   hosts: localhost
   gather_facts: False
 
   tasks:
-    - name: Create vrf within Netbox with only required information
+    - name: Create vrf within NetBox with only required information
       netbox_vrf:
         netbox_url: http://netbox.local
         netbox_token: thisIsMyToken
@@ -164,7 +132,7 @@ EXAMPLES = r"""
 
 RETURN = r"""
 vrf:
-  description: Serialized object as created or already existent within Netbox
+  description: Serialized object as created or already existent within NetBox
   returned: success (when I(state=present))
   type: dict
 msg:
